﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.DataAnnotations;
using System.Collections.ObjectModel;

using XamDataChartNoiseDataDemo.Enums;


namespace XamDataChartNoiseDataDemo.Models
{
    public class CrankshaftSignal : ObservableObject
    {

        #region Private members


        /// <summary>
        /// Delta x-value
        /// </summary>
        private Double _Delta = 0.0011;


        /// <summary>
        /// Signal Name
        /// </summary>
        private String _SignalName;


        /// <summary>
        /// Sensor type (hall, inductive, ...)
        /// </summary>
        private SensorType _SensorType;


        /// <summary>
        /// Signal level (active high, active low, ...)
        /// </summary>
        private SignalLevel _SignalLevel;


        /// <summary>
        /// Crankshaft signal high level
        /// </summary>
        private Double _HighLevel;


        /// <summary>
        /// Crankshaft signal low level
        /// </summary>
        private Double _LowLevel;


        /// <summary>
        /// Number of Teeth
        /// </summary>
        private UInt16 _NrOfTeeth;


        /// <summary>
        /// Tooth width
        /// </summary>
        private Single _ToothWidth;


        /// <summary>
        /// Position of missing teeth
        /// </summary>
        private UInt16?[] _PosMisTeeth;


        /// <summary>
        /// APU 0 degree
        /// </summary>
        private APU0Degree _APU0deg;


        /// <summary>
        /// Delay
        /// </summary>
        private UInt16 _Delay;


        /// <summary>
        /// Number of samples
        /// </summary>
        private UInt32 _NrOfSamples;


        /// <summary>
        /// Signal
        /// </summary>
        private DataPointObservableCollection _Points;


        #endregion


        #region Public members


        /// <summary>
        /// Signal Name
        /// </summary>
        [Required]
        public string SignalName
        {
            get
            {
                return this._SignalName;
            }

            set
            {
                if (this.SetProperty(ref this._SignalName, value))
                {

                }
            }

        }

        /// <summary>
        /// Sensor type (hall, inductive, ...)
        /// </summary>
        [Required]
        public SensorType SensorType
        {
            get
            {
                return this._SensorType;
            }

            set
            {
                if (this.SetProperty(ref this._SensorType, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// Signal level (active high, active low, ...)
        /// </summary>
        [Required]
        public SignalLevel SignalLevel
        {
            get
            {
                return this._SignalLevel;
            }

            set
            {
                if (this.SetProperty(ref this._SignalLevel, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// Crankshaft signal high level
        /// </summary>
        [Required]
        public Double HighLevel
        {
            get
            {
                return this._HighLevel;
            }

            set
            {
                if (this.SetProperty(ref this._HighLevel, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// Crankshaft signal low level
        /// </summary>
        [Required]
        public Double LowLevel
        {
            get
            {
                return this._LowLevel;
            }

            set
            {
                if (this.SetProperty(ref this._LowLevel, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// Number of Teeth
        /// </summary>
        [Required]
        public UInt16 NumberOfTeeth
        {
            get
            {
                return this._NrOfTeeth;
            }

            set
            {
                if (this.SetProperty(ref this._NrOfTeeth, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// Tooth width
        /// </summary>
        [Required]
        public float ToothWidth
        {
            get
            {
                return this._ToothWidth;
            }

            set
            {
                if (this.SetProperty(ref this._ToothWidth, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// Position of missing teeth
        /// </summary>
        [Required]
        public UInt16?[] PositionOfMissingTeeth
        {
            get
            {
                return this._PosMisTeeth;
            }

            set
            {
                if (this.SetProperty(ref this._PosMisTeeth, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// APU 0 degree
        /// </summary>
        [Required]
        public APU0Degree APU0deg
        {
            get
            {
                return this._APU0deg;
            }
            set
            {
                if (this.SetProperty(ref this._APU0deg, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// Delay
        /// </summary>
        [Required]
        public UInt16 Delay
        {
            get
            {
                return this._Delay;
            }

            set
            {
                if (this.SetProperty(ref this._Delay, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// Number of samples used to generate the Crankshaft Signal
        /// </summary>
        [Required]
        public UInt32 NumberOfSamples
        {
            get
            {
                return this._NrOfSamples;
            }

            set
            {
                if (this.SetProperty(ref this._NrOfSamples, value))
                {
                    Generate();
                }
            }

        }


        /// <summary>
        /// Signal
        /// </summary>
        public DataPointObservableCollection Points
        {
            get
            {
                return this._Points;
            }

        }


        #endregion


        #region Constructor


        public CrankshaftSignal(String sigName)
        {

            // default settings
            this._SignalName = sigName;
            this._SensorType = SensorType.Hall;
            this._SignalLevel = SignalLevel.ActiveHigh;
            this._HighLevel = 1;
            this._LowLevel = -1;
            this._NrOfTeeth = 60;
            this._ToothWidth = 50;
            this._PosMisTeeth = new UInt16?[] { 5, 6 };
            this._APU0deg = APU0Degree.Rising;
            this._Delay = 0;
            this._NrOfSamples = 65536;
            this._Points = new DataPointObservableCollection();


            // generate
            Generate();

        }

        #endregion


        #region Public Methods

        /// <summary>
        /// Generate signal
        /// </summary>
        public void GenerateWithSteps()
        {
            this._Points.SupressNotification = true;

            try
            {

                this._Points.Clear();

                Double actHigh;
                Double actLow;
                Double part1Level;
                Double part2Level;
                Double part1width;
                Double highWidth;
                Double toothInt = 360f / this.NumberOfTeeth;
                Double deg = this.Delay;

                if (this.SignalLevel == SignalLevel.ActiveHigh)
                {
                    highWidth = this.ToothWidth;
                }
                else
                {
                    highWidth = 100 - this.ToothWidth;
                }

                for (UInt16 z = 1; z <= this.NumberOfTeeth * 2; z++)
                {

                    if (this.PositionOfMissingTeeth.Contains(z) || this.PositionOfMissingTeeth.Contains((UInt16)(z - this.NumberOfTeeth)))
                    {
                        if (this.SignalLevel == SignalLevel.ActiveHigh)
                        {
                            actHigh = this.LowLevel;
                            actLow = this.LowLevel;
                        }
                        else
                        {
                            actHigh = this.HighLevel;
                            actLow = this.HighLevel;
                        }
                    }
                    else
                    {
                        actHigh = this.HighLevel;
                        actLow = this.LowLevel;
                    }

                    if (APU0deg == APU0Degree.Rising)
                    {
                        part1Level = actHigh;
                        part2Level = actLow;
                        part1width = highWidth;
                    }
                    else
                    {
                        part1Level = actLow;
                        part2Level = actHigh;
                        part1width = (100f - highWidth);
                    }

                    if (z == 1)
                    {
                        //Startpoint
                        this._Points.Add(new DataPoint(deg, part2Level));
                    }

                    //Part 1 startpoint
                    this._Points.Add(new DataPoint(deg, part1Level));

                    //deg forward tooth
                    deg += part1width / 100f * toothInt;

                    //Over 720 due to _Delay => go to beginning
                    if (deg > 720)
                    {

                        //go to start
                        deg -= 720f;

                        //Part 1 endpoint (at 720°)
                        this._Points.Add(new DataPoint(720, part1Level));

                        //Part 1 endpoint (at 0°)
                        this._Points.Add(new DataPoint(0, part1Level));

                    }

                    //Part 1 endpoint
                    this._Points.Add(new DataPoint(deg, part1Level));

                    //Part 2 startpoint
                    this._Points.Add(new DataPoint(deg, part2Level));

                    //deg forward gap
                    deg += (1f - part1width / 100f) * toothInt;

                    //Over 720 due to _Delay => go to beginning
                    if (deg > 720)
                    {

                        //go to start
                        deg -= 720f;

                        //Part 1 endpoint (at 720°)
                        this._Points.Add(new DataPoint(720, part2Level));

                        //Part 1 endpoint (at 0°)
                        this._Points.Add(new DataPoint(0, part2Level));

                    }

                    //Part 2 endpoint
                    this._Points.Add(new DataPoint(deg, part2Level));

                }

                while (this._Points[this._Points.Count - 1].X <= this._Points[0].X)
                {
                    if (this._Points[this._Points.Count - 1].X != this._Points[0].X)
                    {
                        this._Points.Insert(0, new DataPoint(this._Points[this._Points.Count - 1]));
                    }

                    this._Points.RemoveAt(this._Points.Count - 1);
                }

            }
            catch (System.Exception ex)
            {
                throw;
            }
            finally
            {
                this._Points.SupressNotification = false;
            }

        }


        /// <summary>
        /// Generate signal
        /// </summary>
        public void Generate()
        {
            this._Points.SupressNotification = true;

            try
            {

                this._Points.Clear();

                Double actHigh;
                Double actLow;
                Double part1Level;
                Double part2Level;
                Double part1width;
                Double highWidth;
                Double toothInt = 360f / this.NumberOfTeeth;
                Double deg = this.Delay;

                if (this.SignalLevel == SignalLevel.ActiveHigh)
                {
                    highWidth = this.ToothWidth;
                }
                else
                {
                    highWidth = 100 - this.ToothWidth;
                }

                for (UInt16 z = 1; z <= this.NumberOfTeeth * 2; z++)
                {

                    if (this.PositionOfMissingTeeth.Contains(z) || this.PositionOfMissingTeeth.Contains((UInt16)(z - this.NumberOfTeeth)))
                    {
                        if (this.SignalLevel == SignalLevel.ActiveHigh)
                        {
                            actHigh = this.LowLevel;
                            actLow = this.LowLevel;
                        }
                        else
                        {
                            actHigh = this.HighLevel;
                            actLow = this.HighLevel;
                        }
                    }
                    else
                    {
                        actHigh = this.HighLevel;
                        actLow = this.LowLevel;
                    }

                    if (APU0deg == APU0Degree.Rising)
                    {
                        part1Level = actHigh;
                        part2Level = actLow;
                        part1width = highWidth;
                    }
                    else
                    {
                        part1Level = actLow;
                        part2Level = actHigh;
                        part1width = (100f - highWidth);
                    }

                    if (z == 1)
                    {
                        //Startpoint
                        this._Points.Add(new DataPoint(deg, part2Level));
                    }

                    //Part 1 startpoint
                    this._Points.Add(new DataPoint(deg + this._Delta, part1Level));

                    //deg forward tooth
                    deg += part1width / 100f * toothInt;

                    //Over 720 due to _Delay => go to beginning
                    if (deg > 720)
                    {

                        //go to start
                        deg -= 720f;

                        //Part 1 endpoint (at 720°)
                        this._Points.Add(new DataPoint(720, part1Level));

                        //Part 1 endpoint (at 0°)
                        this._Points.Add(new DataPoint(0, part1Level));

                    }

                    //Part 1 endpoint
                    this._Points.Add(new DataPoint(deg, part1Level));

                    //Part 2 startpoint
                    this._Points.Add(new DataPoint(deg + this._Delta, part2Level));

                    //deg forward gap
                    deg += (1f - part1width / 100f) * toothInt;

                    //Over 720 due to _Delay => go to beginning
                    if (deg > 720)
                    {

                        //go to start
                        deg -= 720f;

                        //Part 1 endpoint (at 720°)
                        this._Points.Add(new DataPoint(720, part2Level));

                        //Part 1 endpoint (at 0°)
                        this._Points.Add(new DataPoint(0, part2Level));

                    }

                    //Part 2 endpoint
                    this._Points.Add(new DataPoint(deg, part2Level));

                }

                while (this._Points[this._Points.Count - 1].X <= this._Points[0].X)
                {
                    if (this._Points[this._Points.Count - 1].X != this._Points[0].X)
                    {
                        this._Points.Insert(0, new DataPoint(this._Points[this._Points.Count - 1]));
                    }

                    this._Points.RemoveAt(this._Points.Count - 1);
                }

            }
            catch (System.Exception ex)
            {
                throw;
            }
            finally
            {
                this._Points.SupressNotification = false;
            }

        }

        #endregion


        #region Validation


        #endregion


        #region Overrides


        /// <summary>
        /// ToString()
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return (this.SignalName);
        }


        #endregion




    }
}
